==> aim-transport-0.8.5.tar.gz: aim-transport-0.8.5/src/register.c <==
#include "aimtrans.h"

/*
 * XXX ICK I entered fast mode and just browsed this for errors.  Comb me again
 */
int at_register(jpacket jp)
{
	xmlnode query;
	char *user;
	char *pass;
	session s;
	struct aim_session_t ass;
    xmlnode info;
    xmlnode sn;
    char *key;
    xmlnode dup;
	
    switch(jpacket_subtype(jp))
    {
        case JPACKET__GET:

            log_debug(ZONE, "[AIM] Handling register:get.\n");
	        query = xmlnode_get_tag(jp->x, "query");

            xmlnode_insert_cdata(xmlnode_insert_tag(query,"key"),
                                 jutil_regkey(NULL,jid_full(jp->from)),-1);
            sn = xmlnode_insert_tag(query,"username");
            s = at_session_find_by_jid(jp->from);
            if(s!=NULL)
            {
                ass = s->ass;
                xmlnode_insert_cdata(sn, ass.sn, 
							         strlen(ass.sn));
                xmlnode_insert_tag(query, "registered");
            }
            xmlnode_insert_tag(query,"password");
            info = xmlnode_insert_tag(query, "instructions");
            xmlnode_insert_cdata(info, "Enter your AIM screenname and the password for that account", -1);
            xmlnode_put_attrib(jp->x,"type","result");
            xmlnode_hide_attrib(jp->x,"from");
            xmlnode_hide_attrib(jp->x,"to");

            break;
        case JPACKET__SET:
            log_debug(ZONE, "[AIM] Handling register:set.\n");
            if(xmlnode_get_tag(xmlnode_get_tag(jp->x,"query"),"remove") != NULL)
            {
                char *from,*to;
                at_buddy cur;
                xmlnode sub;

                log_debug(ZONE, "[AIM] Removing registration\n");

                s = at_session_find_by_jid(jp->from);

                if(s == NULL)
                {
                    jutil_error(jp->x, TERROR_REGISTER);
                    break;
                }
                s->exit_flag = 1;
                at_psend(s->mp_to, jp);
                for(cur = s->buddies; cur!=NULL; cur=cur->next)
                {
                    sub = jutil_presnew(JPACKET__UNAVAILABLE, 
                                        jid_full(jp->from),
                                        NULL);
                    xmlnode_put_attrib(sub, "from", jid_full(cur->to));
                    ehandler_send(ehandler_at, sub, jp->from->server);
                }

                sub = jutil_presnew(JPACKET__UNAVAILABLE, jid_full(jp->from), 
                                    NULL);
                xmlnode_put_attrib(sub, "from", jid_full(s->from));
                ehandler_send(ehandler_at, sub, jp->from->server);
                sub = jutil_iqresult(jp->x);
                ehandler_send(ehandler_at,sub,jp->from->server);
            
                return 1;
            }
            /* make sure they sent a valid */
            key = xmlnode_get_tag_data(jp->iq,"key");
            xmlnode_hide(xmlnode_get_tag(jp->iq,"key"));
            if(key == NULL || jutil_regkey(key,jid_full(jp->from)) == NULL)
            {
                jutil_error(jp->x, TERROR_NOTACCEPTABLE);
                break;
            }
            
	    	query = xmlnode_get_tag(jp->x, "query");

		    user = xmlnode_get_data(xmlnode_get_firstchild(
                                    xmlnode_get_tag(query,"username")));
            pass = xmlnode_get_data(xmlnode_get_firstchild(
                                    xmlnode_get_tag(query,"password")));
            xmlnode_hide(xmlnode_get_tag(jp->x,"query"));
            xmlnode_hide_attrib(jp->x,"from");
            xmlnode_hide_attrib(jp->x,"to");
                
            if(user == NULL || pass == NULL)
            {
                xmlnode err;
			
                log_debug(ZONE, "[AIM] Handling register:err.\n");
                xmlnode_put_attrib(jp->x,"type","error");
                err = xmlnode_insert_tag(jp->x,"error");
                xmlnode_put_attrib(err,"code","406"); /*Not Acceptable*/
                xmlnode_insert_cdata(err, 
                                     "You specified an empty user or password",
                                     41 );
                xmlnode_put_attrib(err,"type","invalid");
            }
            else
            {
                xmlnode_put_attrib(jp->x, "type", "result");
                /* XXX Make this work */
                at_auth_subscribe(jp->from, jp->to, user, pass);
            }
            break;
        default:
            log_debug(ZONE, "[AIM] Odd we didn't handle this jpacket");
            break;
    }
    dup = xmlnode_dup(jp->x);
    log_debug(ZONE, "[AIM] Sending %s as iq reply\n", xmlnode2str(dup));
    xmlnode_put_attrib(dup, "to", jid_full(jp->from));
    xmlnode_put_attrib(dup, "from", jid_full(jp->to));
    ehandler_send(ehandler_at, dup, jp->from->server);
	
    return 0;
}

==> aim-transport-0.9.0.tar.gz: aim-transport-0.9.0/src/register.c <==
#include "aimtrans.h"

/*
 * XXX ICK I entered fast mode and just browsed this for errors.  Comb me again
 */
int at_register(ati ti, jpacket jp)
{
	xmlnode query;
	char *user;
	char *pass;
	at_session s;
	struct aim_session_t *ass;
    xmlnode info;
    xmlnode sn;
    char *key;
    xmlnode dup;
	
    switch(jpacket_subtype(jp))
    {
        case JPACKET__GET:

            log_debug(ZONE, "[AIM] Handling register:get.\n");
	        query = xmlnode_get_tag(jp->x, "query");

            xmlnode_insert_cdata(xmlnode_insert_tag(query,"key"),
                                 jutil_regkey(NULL,jid_full(jp->from)),-1);
            sn = xmlnode_insert_tag(query,"username");
            s = at_session_find_by_jid(ti, jp->from);
            if(s!=NULL)
            {
                ass = s->ass;
                xmlnode_insert_cdata(sn, ass->sn, 
							         strlen(ass->sn));
                xmlnode_insert_tag(query, "registered");
            }
            xmlnode_insert_tag(query,"password");
            info = xmlnode_insert_tag(query, "instructions");
            xmlnode_insert_cdata(info, "Enter your AIM screenname and the password for that account", -1);
            xmlnode_put_attrib(jp->x,"type","result");
            jutil_tofrom(jp->x);
           /*
            xmlnode_hide_attrib(jp->x,"from");
            xmlnode_hide_attrib(jp->x,"to");
            */

            break;
        case JPACKET__SET:
            log_debug(ZONE, "[AIM] Handling register:set.\n");
            if(xmlnode_get_tag(xmlnode_get_tag(jp->x,"query"),"remove") != NULL)
            {
                char *from,*to;
                at_buddy cur;
                xmlnode sub;

                log_debug(ZONE, "[AIM] Removing registration\n");

                s = at_session_find_by_jid(ti, jp->from);

                if(s == NULL)
                {
                    jutil_error(jp->x, TERROR_REGISTER);
                    break;
                }
                /* XXX What do I want here?
                 * s->exit_flag = 1;
                 * at_psend(s->mp_to, jp);
                 */
                /*for(cur = s->buddies; cur!=NULL; cur=cur->next)
                {
                    sub = jutil_presnew(JPACKET__UNAVAILABLE, 
                                        jid_full(jp->from),
                                        NULL);
                    xmlnode_put_attrib(sub, "from", jid_full(cur->to));
                    deliver(dpacket_new(sub), ti->i);
                }*/

                sub = jutil_presnew(JPACKET__UNAVAILABLE, jid_full(jp->from), 
                                    NULL);
                xmlnode_put_attrib(sub, "from", jid_full(s->from));
                deliver(dpacket_new(sub), ti->i);
                sub = jutil_iqresult(jp->x);
                deliver(dpacket_new(sub), ti->i);
            
                return 1;
            }
            /* make sure they sent a valid */
            key = xmlnode_get_tag_data(jp->iq,"key");
            xmlnode_hide(xmlnode_get_tag(jp->iq,"key"));
            if(key == NULL || jutil_regkey(key,jid_full(jp->from)) == NULL)
            {
                jutil_error(jp->x, TERROR_NOTACCEPTABLE);
                break;
            }
            
	    	query = xmlnode_get_tag(jp->x, "query");

		    user = xmlnode_get_data(xmlnode_get_firstchild(
                                    xmlnode_get_tag(query,"username")));
            pass = xmlnode_get_data(xmlnode_get_firstchild(
                                    xmlnode_get_tag(query,"password")));
            xmlnode_hide(xmlnode_get_tag(jp->x,"query"));
            xmlnode_hide_attrib(jp->x,"from");
            xmlnode_hide_attrib(jp->x,"to");
                
            if(user == NULL || pass == NULL)
            {
                xmlnode err;
			
                log_debug(ZONE, "[AIM] Handling register:err.\n");
                xmlnode_put_attrib(jp->x,"type","error");
                err = xmlnode_insert_tag(jp->x,"error");
                xmlnode_put_attrib(err,"code","406"); /*Not Acceptable*/
                xmlnode_insert_cdata(err, 
                                     "You specified an empty user or password",
                                     41 );
                xmlnode_put_attrib(err,"type","invalid");
            }
            else
            {
                xmlnode logon;

                logon = xmlnode_new_tag("logon");
                xmlnode_put_attrib(logon, "id", user);
                xmlnode_put_attrib(logon, "pass", pass);
                
                log_debug(ZONE, "[AT] Attempting to start a session from register");

                if(at_session_create(ti, logon, jp)!=NULL)
                {
                    log_debug(ZONE, "[AT] Subscribing to user from register");
                    at_auth_subscribe(ti, jp);

                    at_xdb_set(ti, jp->to->server, jp->from, logon, AT_NS_AUTH);

                    xmlnode_put_attrib(jp->x, "type", "result");
                } else {
                    xmlnode err;
			
                    log_debug(ZONE, "[AT] Unable to start session");
                    xmlnode_put_attrib(jp->x,"type","error");
                    err = xmlnode_insert_tag(jp->x,"error");
                    xmlnode_put_attrib(err,"code","406"); /*Not Acceptable*/
                    xmlnode_insert_cdata(err, 
                                     "Unable to create a session", -1);
                    xmlnode_put_attrib(err,"type","invalid");
                }
            }
            break;
        default:
            log_debug(ZONE, "[AIM] Odd we didn't handle this jpacket for subtype %d", jpacket_subtype(jp));
            return 0;
    }
    dup = xmlnode_dup(jp->x);
    log_debug(ZONE, "[AIM] Sending %s as iq reply\n", xmlnode2str(dup));
    deliver(dpacket_new(dup), ti->i);
	
    return 0;
}
